'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _testcafeBrowserTools = require('testcafe-browser-tools');

var _testcafeBrowserTools2 = _interopRequireDefault(_testcafeBrowserTools);

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _message = require('../../../warnings/message');

var _message2 = _interopRequireDefault(_message);

var _delay = require('../../../utils/delay');

var _delay2 = _interopRequireDefault(_delay);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*eslint-disable no-undef*/
function getTitle() {
    return document.title;
}

function getCurrentSize() {
    return {
        width: window.innerWidth,
        height: window.innerHeight
    };
}
/*eslint-disable no-undef*/

var GET_TITLE_SCRIPT = getTitle.toString();
var GET_CURRENT_SIZE_SCRIPT = getCurrentSize.toString();

var BROWSER_OPENING_DELAY = 2000;

var RESIZE_DIFF_SIZE = {
    width: 100,
    height: 100
};

function sumSizes(sizeA, sizeB) {
    return {
        width: sizeA.width + sizeB.width,
        height: sizeA.height + sizeB.height
    };
}

function subtractSizes(sizeA, sizeB) {
    return {
        width: sizeA.width - sizeB.width,
        height: sizeA.height - sizeB.height
    };
}

var BrowserProviderBase = function () {
    function BrowserProviderBase() {
        (0, _classCallCheck3.default)(this, BrowserProviderBase);

        // HACK: The browser window has different border sizes in normal and maximized modes. So, we need to be sure that the window is
        // not maximized before resizing it in order to keep the mechanism of correcting the client area size working. When browser is started,
        // we are resizing it for the first time to switch the window to normal mode, and for the second time - to restore the client area size.

        this.resizeCorrections = {};
    }

    BrowserProviderBase.prototype.calculateResizeCorrections = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(browserId) {
            var title, currentSize, etalonSize, resizedSize, correctionSize;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return this.waitForConnectionReady(browserId);

                        case 2:
                            _context.next = 4;
                            return (0, _delay2.default)(BROWSER_OPENING_DELAY);

                        case 4:
                            _context.next = 6;
                            return this.runInitScript(browserId, GET_TITLE_SCRIPT);

                        case 6:
                            title = _context.sent;
                            _context.next = 9;
                            return _testcafeBrowserTools2.default.isMaximized(title);

                        case 9:
                            if (_context.sent) {
                                _context.next = 11;
                                break;
                            }

                            return _context.abrupt('return');

                        case 11:
                            _context.next = 13;
                            return this.runInitScript(browserId, GET_CURRENT_SIZE_SCRIPT);

                        case 13:
                            currentSize = _context.sent;
                            etalonSize = subtractSizes(currentSize, RESIZE_DIFF_SIZE);
                            _context.next = 17;
                            return _testcafeBrowserTools2.default.resize(title, currentSize.width, currentSize.height, etalonSize.width, etalonSize.height);

                        case 17:
                            _context.next = 19;
                            return this.runInitScript(browserId, GET_CURRENT_SIZE_SCRIPT);

                        case 19:
                            resizedSize = _context.sent;
                            correctionSize = subtractSizes(resizedSize, etalonSize);
                            _context.next = 23;
                            return _testcafeBrowserTools2.default.resize(title, resizedSize.width, resizedSize.height, etalonSize.width, etalonSize.height);

                        case 23:
                            _context.next = 25;
                            return this.runInitScript(browserId, GET_CURRENT_SIZE_SCRIPT);

                        case 25:
                            resizedSize = _context.sent;


                            correctionSize = sumSizes(correctionSize, subtractSizes(resizedSize, etalonSize));

                            this.resizeCorrections[browserId] = correctionSize;

                            _context.next = 30;
                            return _testcafeBrowserTools2.default.maximize(title);

                        case 30:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function calculateResizeCorrections(_x) {
            return _ref.apply(this, arguments);
        }

        return calculateResizeCorrections;
    }();

    BrowserProviderBase.prototype.resizeWindow = function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(browserId, width, height, currentWidth, currentHeight) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (!_osFamily2.default.linux) {
                                _context2.next = 3;
                                break;
                            }

                            this.reportWarning(browserId, _message2.default.browserManipulationsNotSupportedOnLinux);
                            return _context2.abrupt('return');

                        case 3:

                            if (this.resizeCorrections[browserId]) {
                                width -= this.resizeCorrections[browserId].width;
                                height -= this.resizeCorrections[browserId].height;

                                delete this.resizeCorrections[browserId];
                            }

                            _context2.next = 6;
                            return _testcafeBrowserTools2.default.resize(browserId, currentWidth, currentHeight, width, height);

                        case 6:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function resizeWindow(_x2, _x3, _x4, _x5, _x6) {
            return _ref2.apply(this, arguments);
        }

        return resizeWindow;
    }();

    BrowserProviderBase.prototype.takeScreenshot = function () {
        var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(browserId, screenshotPath) {
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            if (!_osFamily2.default.linux) {
                                _context3.next = 3;
                                break;
                            }

                            this.reportWarning(browserId, _message2.default.browserManipulationsNotSupportedOnLinux);
                            return _context3.abrupt('return');

                        case 3:
                            _context3.next = 5;
                            return _testcafeBrowserTools2.default.screenshot(browserId, screenshotPath);

                        case 5:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this);
        }));

        function takeScreenshot(_x7, _x8) {
            return _ref3.apply(this, arguments);
        }

        return takeScreenshot;
    }();

    return BrowserProviderBase;
}();

exports.default = BrowserProviderBase;
module.exports = exports['default'];