'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _promisifyEvent = require('promisify-event');

var _promisifyEvent2 = _interopRequireDefault(_promisifyEvent);

var _mapReverse = require('map-reverse');

var _mapReverse2 = _interopRequireDefault(_mapReverse);

var _path = require('path');

var _events = require('events');

var _lodash = require('lodash');

var _bootstrapper = require('./bootstrapper');

var _bootstrapper2 = _interopRequireDefault(_bootstrapper);

var _reporter2 = require('../reporter');

var _reporter3 = _interopRequireDefault(_reporter2);

var _task = require('./task');

var _task2 = _interopRequireDefault(_task);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_SELECTOR_TIMEOUT = 10000;

var Runner = function (_EventEmitter) {
    (0, _inherits3.default)(Runner, _EventEmitter);

    function Runner(proxy, browserConnectionGateway) {
        (0, _classCallCheck3.default)(this, Runner);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.proxy = proxy;
        _this.bootstrapper = new _bootstrapper2.default(browserConnectionGateway);
        _this.pendingTaskPromises = [];

        _this.opts = {
            screenshotPath: null,
            takeScreenshotsOnFails: false,
            skipJsErrors: false,
            quarantineMode: false,
            reportOutStream: void 0,
            selectorTimeout: DEFAULT_SELECTOR_TIMEOUT
        };
        return _this;
    }

    Runner._disposeTaskAndBrowsers = function () {
        var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(task, browserSet) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            task.abort();
                            task.removeAllListeners();

                            _context.next = 4;
                            return browserSet.dispose();

                        case 4:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function _disposeTaskAndBrowsers(_x, _x2) {
            return _ref.apply(this, arguments);
        }

        return _disposeTaskAndBrowsers;
    }();

    Runner.prototype._createCancelablePromise = function _createCancelablePromise(taskPromise) {
        var _this2 = this;

        var promise = taskPromise.then(function (_ref2) {
            var completionPromise = _ref2.completionPromise;
            return completionPromise;
        });
        var removeFromPending = function removeFromPending() {
            return (0, _lodash.pull)(_this2.pendingTaskPromises, promise);
        };

        promise.then(removeFromPending).catch(removeFromPending);

        promise.cancel = function () {
            return taskPromise.then(function (_ref3) {
                var cancelTask = _ref3.cancelTask;
                return cancelTask();
            }).then(removeFromPending);
        };

        this.pendingTaskPromises.push(promise);
        return promise;
    };

    // Run task


    Runner.prototype._getTaskResult = function () {
        var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(task, browserSet, reporter) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            task.on('browser-job-done', function (job) {
                                return browserSet.releaseConnection(job.browserConnection);
                            });

                            _context2.prev = 1;
                            _context2.next = 4;
                            return _pinkie2.default.race([(0, _promisifyEvent2.default)(task, 'done'), (0, _promisifyEvent2.default)(browserSet, 'error')]);

                        case 4:
                            _context2.next = 11;
                            break;

                        case 6:
                            _context2.prev = 6;
                            _context2.t0 = _context2['catch'](1);
                            _context2.next = 10;
                            return Runner._disposeTaskAndBrowsers(task, browserSet);

                        case 10:
                            throw _context2.t0;

                        case 11:
                            _context2.next = 13;
                            return browserSet.dispose();

                        case 13:
                            return _context2.abrupt('return', reporter.testCount - reporter.passed);

                        case 14:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this, [[1, 6]]);
        }));

        function _getTaskResult(_x3, _x4, _x5) {
            return _ref4.apply(this, arguments);
        }

        return _getTaskResult;
    }();

    Runner.prototype._runTask = function _runTask(reporterPlugin, browserSet, tests) {
        var _this3 = this;

        var completed = false;
        var task = new _task2.default(tests, browserSet.connections, this.proxy, this.opts);
        var reporter = new _reporter3.default(reporterPlugin, task, this.opts.reportOutStream);
        var completionPromise = this._getTaskResult(task, browserSet, reporter);

        var setCompleted = function setCompleted() {
            completed = true;
        };

        completionPromise.then(setCompleted).catch(setCompleted);

        var cancelTask = function () {
            var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
                return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                        switch (_context3.prev = _context3.next) {
                            case 0:
                                if (completed) {
                                    _context3.next = 3;
                                    break;
                                }

                                _context3.next = 3;
                                return Runner._disposeTaskAndBrowsers(task, browserSet);

                            case 3:
                            case 'end':
                                return _context3.stop();
                        }
                    }
                }, _callee3, _this3);
            }));

            return function cancelTask() {
                return _ref5.apply(this, arguments);
            };
        }();

        return { completionPromise: completionPromise, cancelTask: cancelTask };
    };

    // API


    Runner.prototype.src = function src() {
        for (var _len = arguments.length, sources = Array(_len), _key = 0; _key < _len; _key++) {
            sources[_key] = arguments[_key];
        }

        sources = (0, _lodash.flattenDeep)(sources).map(function (path) {
            return (0, _path.resolve)(path);
        });

        this.bootstrapper.sources = this.bootstrapper.sources.concat(sources);

        return this;
    };

    Runner.prototype.browsers = function browsers() {
        for (var _len2 = arguments.length, _browsers = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            _browsers[_key2] = arguments[_key2];
        }

        this.bootstrapper.browsers = this.bootstrapper.browsers.concat((0, _lodash.flattenDeep)(_browsers));

        return this;
    };

    Runner.prototype.reporter = function reporter(_reporter, outStream) {
        this.bootstrapper.reporter = _reporter;
        this.opts.reportOutStream = outStream;

        return this;
    };

    Runner.prototype.filter = function filter(_filter) {
        this.bootstrapper.filter = _filter;

        return this;
    };

    Runner.prototype.screenshots = function screenshots(path) {
        var takeOnFails = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        this.opts.takeScreenshotsOnFails = takeOnFails;
        this.opts.screenshotPath = path;

        return this;
    };

    Runner.prototype.run = function run() {
        var _this4 = this;

        var _ref6 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        var skipJsErrors = _ref6.skipJsErrors;
        var quarantineMode = _ref6.quarantineMode;
        var selectorTimeout = _ref6.selectorTimeout;

        this.opts.skipJsErrors = !!skipJsErrors;
        this.opts.quarantineMode = !!quarantineMode;
        this.opts.selectorTimeout = selectorTimeout === void 0 ? DEFAULT_SELECTOR_TIMEOUT : selectorTimeout;

        var runTaskPromise = this.bootstrapper.createRunnableConfiguration().then(function (_ref7) {
            var reporterPlugin = _ref7.reporterPlugin;
            var browserSet = _ref7.browserSet;
            var tests = _ref7.tests;

            _this4.emit('done-bootstrapping');

            return _this4._runTask(reporterPlugin, browserSet, tests);
        });

        return this._createCancelablePromise(runTaskPromise);
    };

    Runner.prototype.stop = function () {
        var _ref8 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
            var cancellationPromises;
            return _regenerator2.default.wrap(function _callee4$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            // NOTE: When taskPromise is cancelled, it is removed from
                            // the pendingTaskPromises array, which leads to shifting indexes
                            // towards the beginning. So, we must copy the array in order to iterate it,
                            // or we can perform iteration from the end to the beginning.
                            cancellationPromises = (0, _mapReverse2.default)(this.pendingTaskPromises, function (taskPromise) {
                                return taskPromise.cancel();
                            });
                            _context4.next = 3;
                            return _pinkie2.default.all(cancellationPromises);

                        case 3:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, _callee4, this);
        }));

        function stop() {
            return _ref8.apply(this, arguments);
        }

        return stop;
    }();

    return Runner;
}(_events.EventEmitter);

exports.default = Runner;
module.exports = exports['default'];